#ifndef OraclePixelGeoManager_H
#define OraclePixelGeoManager_H

#include <string>
#include <iostream>
#include <sstream>
#include <map>
#include "GeoModelKernel/GeoMaterial.h"
#include "CLHEP/Geometry/Vector3D.h"
#include "PixelGeoModel/PixelGeometryManager.h"
#include "RDBAccessSvc/IRDBRecord.h"
#include "RDBAccessSvc/IRDBRecordset.h"

class StoredMaterialManager;
class StoreGateSvc;



namespace InDetDD {
  class PixelDetectorManager;
  class SiCommonItems;
}

class OraclePixGeoManager : public PixelGeometryManager {
 private:
  //
  // NovaObjects: PixbGeo
  //
  const IRDBRecordset * atls;
  const IRDBRecordset * PixelBarrelGeneral;
  const IRDBRecordset * PixelBarrelService;
  const IRDBRecordset * PixelCommon;
  const IRDBRecordset * PixelDisk;
  const IRDBRecordset * PixelEndcapGeneral;
  const IRDBRecordset * PixelEndcapService;
  const IRDBRecordset * PixelLayer;
  const IRDBRecordset * PixelModule;
  const IRDBRecordset * PixelStave;
  //const IRDBRecordset * pxbo;
  const IRDBRecordset * pxbi;
  const IRDBRecordset * pbrn; //JBdVtemporary
  const IRDBRecordset * pbac; //JBdVtemporary
  //
  // Detailed ladder and services on ladder
  //
  const IRDBRecordset * ptla;
  const IRDBRecordset * pctr;
  const IRDBRecordset * pftr;
  const IRDBRecordset * pttr;
  const IRDBRecordset * pome;
  const IRDBRecordset * poti;
  const IRDBRecordset * pobi;
  const IRDBRecordset * poai;
  const IRDBRecordset * poci;
  const IRDBRecordset * posi;
  const IRDBRecordset * pccf;
  const IRDBRecordset * pcff;
  //
  // NovaObjects: PixeGeo
  //
  const IRDBRecordset * pxei;
  const IRDBRecordset * pefi;
  const IRDBRecordset * peoi;
  //
  // NovaObjects: Pixbdig
  //
  const IRDBRecordset * pdch;
  const IRDBRecordset * pxbd;
  const IRDBRecordset * plor;
  const IRDBRecordset * plrn;
  
  // eta, phi, layer/disk, side (EC)
  int m_eta,m_phi;
  int currentLD;
  int BarrelEndcap;
  int _side;

  int currentTrap;
  int currentCyliEl;
  int currentParaEl;
  std::string currentParaElType;
  std::string currentTrapType;

  // le poids
  int    m_nechelles;
  double m_lepoids;
  double m_weiomega;
  double m_weicable;
  double m_weimod;
  double m_weitmt;
  double m_weifluid;

  int m_barrelInFrames, m_endcapInFrames;

  // flag to build also the services
  bool m_services;
  bool m_servicesOnLadder;

  // flag to make elements compatible with G3 digits
  bool m_g3CompatibleDigits;

  // flag to indicate initial layout (2nd layer and disk missing)
  bool m_initialLayout;
  
  // flag to force B-layer to have 300um pixels and be 200um thick
  bool m_dc1Geometry;

  // control whether callbacks get registered 
  bool m_alignable;

  // Magnetic Field
  bool m_magFieldFromNova;
  bool m_useMagFieldSvc;
  mutable HepVector3D m_magField;
  
  // Class holding items that only one instance is needed for all detector elements.
  InDetDD::SiCommonItems * m_commonItems;

  // the det descr manager...
  InDetDD::PixelDetectorManager *m_pDDmgr;

  //the material manager
  StoredMaterialManager* m_pMatMgr;

  // The Transient Detector Store Service
  StoreGateSvc* m_pDetStore;   

  // The name of the collections in the det store
  std::string m_elementsObjectName;
  std::string m_designsObjectName;

  // The design of the pixel modules. 
  std::string m_barrelDesignName;
  std::string m_endcapDesignName;
  std::string m_blayerDesignName;

  // version tag
  std::string m_versionTag;

 private:

  double CalculateThickness(double,std::string);

 public:

  OraclePixGeoManager();

  //
  // GET THE OTHER MANAGERS FROM STOREGATE
  // -------------------------------------

  // Get the material manager:
  StoredMaterialManager* getMaterialManager();

  // PixelDetectorManager
  InDetDD::PixelDetectorManager *GetPixelDDManager();

  // 
  // VERSION TAG
  // -----------
  virtual std::string versionTag() const {return m_versionTag;}

  //
  // DESIGN AND ELEMENT NAMES
  // ------------------------
  void SetDetElementName(std::string);
  std::string GetDetElementName();
  void SetDesignName(std::string);
  std::string GetDesignName();

  void SetBarrelModuleName(std::string);
  std::string GetBarrelModuleName();
  void SetEndcapModuleName(std::string);
  std::string GetEndcapModuleName();
  void SetLayer0ModuleName(std::string);
  std::string GetLayer0ModuleName();

  //
  // BUILDING DEFINITIONS
  // --------------------

  // Do I want the services?
  void SetServices(bool isservice) {m_services = isservice;};
  bool DoServices();

  //JBdV Do I want the services? (Omega + Al tube + cables T0 + pigtails + connectors)
  void SetServicesOnLadder(bool isservice) {m_servicesOnLadder = isservice;};
  bool DoServicesOnLadder();

  // Make elements compatible with G3 digits
  void SetG3CompatibleDigits(bool flag) {m_g3CompatibleDigits = flag;}
  bool G3CompatibleDigits() const;

  // Initial layout (2nd layer missing)
  void SetInitialLayout(bool flag) {m_initialLayout = flag;}
  bool InitialLayout() const;

  // DC1 Geometry. 300 um long pixels and 200 um thick sensor in B layer. 
  void SetDC1Geometry(bool flag) {m_dc1Geometry = flag;}
  bool DC1Geometry() const;

  // Control whether callbacks get registered
  void SetAlignable(bool flag) {m_alignable = flag;}
  bool Alignable() const;

  //
  // BUILDER HELPERS
  // ----------------

  void SetEta(int eta) {m_eta = eta;};
  void SetPhi(int phi) {m_phi = phi;};
  int Eta() {return m_eta;};
  int Phi() {return m_phi;};

  // What am I building?
  bool isBarrel();
  bool isEndcap();
  void SetBarrel();
  void SetEndcap();

  // The layer/disk barrel/endcap can be changed by these function.
  void SetCurrentLD(int i);
  int GetLD() {return currentLD;};

  // Which layers/disks are present?
  bool isLDPresent();

  // The side
  void SetPos() {_side = 1;};
  void SetNeg() {_side = -1;};
  int GetSide() {return _side;};

  // The TMT and the Omega
  void SetCurrentTrap(int i) {currentTrap = i;};
  int GetTrap() {return currentTrap;};
  void SetCurrentParaEl(int i) {currentParaEl = i;};
  int GetParaEl() {return currentParaEl;};
  void SetCurrentParaElType(std::string type) {currentParaElType = type;};
  std::string GetParaElType() {return currentParaElType;};
  void SetCurrentTrapType(std::string type) {currentTrapType = type;};
  std::string GetTrapType() {return currentTrapType;};
  void SetCurrentCyliEl(int i) {currentCyliEl = i;};
  int GetCyliEl() {return currentCyliEl;};

  // the weights (for checking...)
  void AddNStaves(unsigned int i) {m_nechelles+= i;};
  unsigned int NStave() {return m_nechelles;};
  void AddWeight(double p) {m_lepoids+= p;};
  double LePoids() {return m_lepoids;};
  void AddWeightOmega(double p) {m_weiomega+= p;};
  double WeiOmega() {return m_weiomega;};
  void AddWeightTrapTmt(double p) {m_weitmt+= p;};
  double WeiTrapTmt() {return m_weitmt;};
  void AddWeightModule(double p) {m_weimod+= p;};
  double WeiModule() {return m_weimod;};
  void AddWeightCable(double p) {m_weicable+= p;};
  double WeiCable() {return m_weicable;};
  void AddWeightFluid(double p) {m_weifluid+= p;};
  double WeiFluid() {return m_weifluid;};  

  //
  // DETECTOR PARAMTERS
  // ------------------


  // Version Number, for the Barrel/EndCap
  inline int PixelBarrelMajorVersion();
  inline int PixelBarrelMinorVersion();
  inline int PixelEndcapMajorVersion();
  inline int PixelEndcapMinorVersion();  

  // Si Board
  double PixelBoardWidth();
  double PixelBoardLength();
  double PixelBoardThickness();
  double PixelBoardActiveLen();

  // Hybrid
  double PixelHybridWidth();
  double PixelHybridLength();
  double PixelHybridThickness();

  //  Fe Chips (PBRN)
  double PixelChipWidth();
  double PixelChipLength();
  double PixelChipGap();
  double PixelChipThickness();

  // Disk Carbon Structure
  double PixelECCarbonRMin(std::string);
  double PixelECCarbonRMax(std::string);
  double PixelECCarbonThickness(std::string);
  std::string PixelECCarbonMaterial(std::string);

  // Services
  double* PixelServiceR(std::string, int );
  double* PixelServiceZ(std::string, int );
  std::string PixelServiceMaterial(std::string, int );
  int PixelServiceLD(std::string, int );
  int PixelServiceNFrame(std::string);

  //  Atlas Global volume (from ATLS)
  inline double GetATLSRadius();
  inline double GetATLSRmin();
  inline double GetATLSLength();

  //  Pixel container (from PXBG)
  inline double PixelRMin();
  inline double PixelRMax();
  inline double PixelHalfLength();
  inline int PixelBarrelNLayer();

  // Pixel Barrel  (from PXBO)
  inline double PixelBarrelRMin();
  inline double PixelBarrelRMax();
  inline double PixelBarrelHalfLength();
  
  // Pixel Layers Geomtry (PXBI)
  inline double PixelLayerRadius();
  inline double PixelLadderHalfLength();
  inline double PixelLadderWidth();
  double PixelLadderThickness();
  inline double PixelLadderTilt();
  inline int NPixelSectors();
  inline double PixelBalcony();
  inline int PixelNModule();
  inline double PixelModuleDrDistance();
  inline double PixelModuleAngle();
  inline double PixelModulePosition(int);
  inline double PixelModuleShiftFlag(int);
  inline double PixelModuleAngleSign(int);
  inline double PixelGapDetLad();

  // Barrel LAYER CABLES (PBAC)
  inline double PixelCableWidth();
  inline double PixelCableThickness();
  inline double PixelCableZMax();
  inline double PixelCableZMin();
  inline double PixelCableDeltaZ();

  // trapezoid parameters
  inline int PixelNTrap();
  inline int PixelNTrapC();
  inline int PixelNTrapE();
  inline double PixelTrapWidth();
  inline double PixelTrapDzdr();
  inline double PixelTrapHalfLength();
  inline double PixelTrapHalfWidthY();
  inline double PixelTrapHalfWidthXzn();
  inline double PixelTrapHalfWidthXzp();
  inline double PixelTrapPosx();
  inline double PixelTrapPosy();
  inline double PixelTrapPosz();
  inline int PixelTrapLabel();
  inline int PixelTrapMat();

  // Omega parameters
  inline int PixelOmegaNParaEl();
  inline int PixelOmegaNCyliEl();
  inline double PixelPosiHalfLength();
  inline double PixelPosiWidth();
  inline double PixelPosiThick();
  inline double PixelPosiTilt();
  inline double PixelPosiPosx();
  inline double PixelPosiPosy();
  inline double PixelPosiPosz();
  inline double PixelPosiRotz();
  inline int PixelPosiLabel();
  inline double PixelPotiHalfLength(std::string);
  inline double PixelPotiRmin(std::string);
  inline double PixelPotiRmax(std::string);
  inline double PixelPotiPhi1(std::string);
  inline double PixelPotiPhi2(std::string);
  inline double PixelPotiPosx(std::string);
  inline double PixelPotiPosy(std::string);
  inline double PixelPotiPosz(std::string);
  inline int PixelPotiLabel(std::string);
  inline int PixelConnectorANParaEl();
  inline int PixelConnectorCNParaEl();
  inline int PixelConnectorBNCyliEl();
  inline int PixelMaterialPara();
  inline int PixelMaterialCyli(std::string);
  std::string PixelGetMat(int);
  inline int PixelFluidOrient();

  // Pixel Endcap PXEG
  inline int PixelEndcapNDisk();

  // Pixel Endcap Container PEVO
  inline double PixelEndcapRMin();
  inline double PixelEndcapRMax();
  inline double PixelEndcapZMin();
  inline double PixelEndcapZMax();
  inline int PixelEndcapNSupportFrames();

  // Pixel Disks PXEI
  inline double PixelDiskPosition();
  inline double PixelECSiDz1();
  inline double PixelECSiDz2();
  inline double PixelDiskRMin();
  inline int PixelECNSectors1();
  inline int PixelECNSectors2();

  // Endcap CABLES (PEAC)
  inline double PixelECCablesRMin();
  inline double PixelECCablesRMax();
  double PixelECCablesThickness();
  inline double PixelECCablesDistance();

  // Design Parameter
  //
  inline double DesignRPActiveArea();
  inline double DesignZActiveArea();

  inline int DesignCircuitsPerColumn();
  inline int DesignCircuitsPerRow();

  int DesignCellColumnsPerCircuit(bool isBLayer);
  int DesignCellRowsPerCircuit(bool isBLayer);
  int DesignDiodeColumnsPerCircuit(bool isBLayer);
  int DesignDiodeRowsPerCircuit(bool isBLayer);

  inline double DesignPitchRP(bool isBLayer);
  inline double DesignGapRP();
  inline double DesignPitchZ(bool isBLayer);
  inline double DesignGapZ();

  // Ganged Pixels
  int NumberOfEmptyRows();
  int EmptyRows(int index);
  int EmptyRowConnections(int index);

  // 
  // conditions stuff (probably it shouldnt be here)
  double Voltage(bool isBLayer);
  double Temperature(bool isBLayer);

  // Magnetic field
  const HepVector3D & magneticField(bool isBLayer = false) const;
  void setMagneticField(const HepVector3D & field);
  bool useMagneticFieldSvc() const;
  void setUseMagneticFieldSvc(bool flag);  

  // CommonItems for Det Elements
  InDetDD::SiCommonItems * commonItems() const;
  void setCommonItems(InDetDD::SiCommonItems * commonItems); 


};

//
// INLINE METHODS:
//

// ATLS
double OraclePixGeoManager::GetATLSRadius() 
{
  return (*atls)[0]->getDouble("RMAX")*cm;
 }

double OraclePixGeoManager::GetATLSRmin() 
{
 return (*atls)[0]->getDouble("RMIN")*cm;
}
double OraclePixGeoManager::GetATLSLength() 
{
  return (*atls)[0]->getDouble("ZMAX")*cm;
}

//
// Version of the Geometry
//

// PXPA
int OraclePixGeoManager::PixelBarrelMajorVersion() { return static_cast<int>((*PixelBarrelGeneral)[0]->getDouble("VERSION"));}
int OraclePixGeoManager::PixelBarrelMinorVersion()
  { return static_cast<int>(((*PixelBarrelGeneral)[0]->getDouble("VERSION") - PixelBarrelMajorVersion())*10 + 0.5);}
int OraclePixGeoManager::PixelEndcapMajorVersion() { return static_cast<int>((*PixelEndcapGeneral)[0]->getDouble("VERSION"));}
int OraclePixGeoManager::PixelEndcapMinorVersion() 
  { return static_cast<int>(((*PixelEndcapGeneral)[0]->getDouble("VERSION") - PixelEndcapMajorVersion())*10 + 0.5);}

// PXBG
double OraclePixGeoManager::PixelRMin() {return (*PixelCommon)[0]->getDouble("RMIN")*cm;}
double OraclePixGeoManager::PixelRMax() {return (*PixelCommon)[0]->getDouble("RMAX")*cm;}
double OraclePixGeoManager::PixelHalfLength() {return (*PixelCommon)[0]->getDouble("HALFLENGTH")*cm;}
int OraclePixGeoManager::PixelBarrelNLayer() {return (*PixelBarrelGeneral)[0]->getInt("NLAYER");}

// PixelBarrelGeneral
double OraclePixGeoManager::PixelBarrelRMin() {return (*PixelBarrelGeneral)[0]->getDouble("RMIN")*cm;}
double OraclePixGeoManager::PixelBarrelRMax() {return (*PixelBarrelGeneral)[0]->getDouble("RMAX")*cm;}
double OraclePixGeoManager::PixelBarrelHalfLength() {return (*PixelBarrelGeneral)[0]->getDouble("HALFLENGTH")*cm;}

// PXBI
double OraclePixGeoManager::PixelLayerRadius() 
{
  return (*PixelLayer)[currentLD]->getDouble("RLAYER")*cm;
}

double OraclePixGeoManager::PixelLadderHalfLength() 
{
  //JBdV temporary return (*PixelStave)[0]->getDouble("SUPPORTHLENGTH")*cm;
  return (*pxbi)[0]->getDouble("DZLADDER")*cm;
}

double OraclePixGeoManager::PixelLadderWidth() 
{
  //JBdV temporary return (*PixelStave)[0]->getDouble("SUPPORTWIDTH")*cm;
  return (*pxbi)[0]->getDouble("DYLADDER")*cm;
}

double OraclePixGeoManager::PixelLadderTilt() 
{
  return (*PixelLayer)[currentLD]->getDouble("STAVETILT")*deg;
}
int OraclePixGeoManager::NPixelSectors() 
{
  return (*PixelLayer)[currentLD]->getInt("NSECTORS");
}

double OraclePixGeoManager::PixelBalcony() 
{
  return (*pxbi)[currentLD]->getDouble("DXELEB")*cm;
}

int OraclePixGeoManager::PixelNModule() 
{
  return (*PixelStave)[0]->getInt("NMODULE");
}

double OraclePixGeoManager::PixelModuleAngle() 
{
  return (*PixelStave)[0]->getDouble("MODULETILT")*deg;
}

double OraclePixGeoManager::PixelModuleDrDistance() 
{
  return (*PixelStave)[0]->getDouble("CENTRMODULESHIFT")*cm;
}

double OraclePixGeoManager::PixelModulePosition(int im) 
{
  return (*PixelStave)[0]->getDouble("MODULEDZ")*cm*im;
};

// OBSOLETE!!! TO MOVE INTO THE NEW FACTORY 
double OraclePixGeoManager::PixelModuleShiftFlag(int im) 
{
  if(im == 0) return 1.;
  return 0.;
};

// OBSOLETE!!! TO MOVE INTO THE NEW FACTORY 
double OraclePixGeoManager::PixelModuleAngleSign(int im) 
{
  if(im < 6) return 1.;
  if(im > 6) return -1.;
  return 0.;
};

double OraclePixGeoManager::PixelGapDetLad()
{
  //JBdV return 0.;
  return (*pxbi)[currentLD]->getDouble("XGAPLADD")*cm;
};



// PBAC
double OraclePixGeoManager::PixelCableWidth() {
  return (*PixelStave)[0]->getDouble("CABLEWIDTH")*cm;
};

double OraclePixGeoManager::PixelCableThickness() {
  //JBdV temporary return (*PixelStave)[0]->getDouble("CABLETHICK")*cm;
  return (*pbac)[0]->getDouble("XCABLE")*cm;
};

double OraclePixGeoManager::PixelCableZMax() {
  return (*PixelStave)[0]->getDouble("SUPPORTHLENGTH")*cm;
};

double OraclePixGeoManager::PixelCableZMin() {
  return (*PixelStave)[0]->getDouble("CABLEZMIN")*cm;
};

double OraclePixGeoManager::PixelCableDeltaZ() {
  return (*PixelStave)[0]->getDouble("MODULEDZ")*cm;
};


// EndCap PXEG  Mistake in table name.
int OraclePixGeoManager::PixelEndcapNDisk() {return (*PixelEndcapGeneral)[0]->getInt("NDISK");};

// Endcap container PEVO
double  OraclePixGeoManager::PixelEndcapRMin() {return (*PixelEndcapGeneral)[0]->getDouble("RMIN")*cm;};

double  OraclePixGeoManager::PixelEndcapRMax() {return (*PixelEndcapGeneral)[0]->getDouble("RMAX")*cm;};

double  OraclePixGeoManager::PixelEndcapZMin() {return (*PixelEndcapGeneral)[0]->getDouble("ZMIN")*cm;};

double  OraclePixGeoManager::PixelEndcapZMax() {return (*PixelEndcapGeneral)[0]->getDouble("ZMAX")*cm;};

int OraclePixGeoManager::PixelEndcapNSupportFrames() {return (int) (*PixelEndcapGeneral)[0]->getDouble("NFRAME");};

// Endcap Inner (PXEI)
double  OraclePixGeoManager::PixelDiskPosition() {return (*PixelDisk)[currentLD]->getDouble("ZDISK")*cm;};

double  OraclePixGeoManager::PixelDiskRMin() {return (*PixelDisk)[currentLD]->getDouble("RIDISK")*cm;};

double OraclePixGeoManager::PixelECSiDz1() {return (*PixelDisk)[currentLD]->getDouble("DZCOUNTER")*cm;};

double OraclePixGeoManager::PixelECSiDz2() {return (*PixelDisk)[currentLD]->getDouble("DZCOUNTER")*cm;};

int OraclePixGeoManager::PixelECNSectors1() {return (*PixelDisk)[currentLD]->getInt("NMODULE");};

int OraclePixGeoManager::PixelECNSectors2() {return (*PixelDisk)[currentLD]->getInt("NMODULE");};

// Endcap Cables PEAC
double OraclePixGeoManager::PixelECCablesRMin() {
  return (*PixelDisk)[currentLD]->getDouble("RMINCABLE")*cm;
};

double OraclePixGeoManager::PixelECCablesRMax() {
  return (*PixelDisk)[currentLD]->getDouble("RMAXCABLE")*cm;
};


double OraclePixGeoManager::PixelECCablesDistance() {
  return (*PixelDisk)[currentLD]->getDouble("ZCABLE")*cm;
};
//
// Design
//

double OraclePixGeoManager::DesignRPActiveArea(){
  return (*pxbi)[0]->getDouble("DYACTIVE")*cm;
}
double OraclePixGeoManager::DesignZActiveArea(){
  return (*pxbi)[0]->getDouble("DZELEB")*cm;
}

double OraclePixGeoManager::DesignPitchRP(bool isBLayer) {
  if(isBLayer) return (*pxbd)[0]->getDouble("PITCHRP")*cm;
  else return (*pxbd)[1]->getDouble("PITCHRP")*cm;
}

double OraclePixGeoManager::DesignPitchZ(bool isBLayer) {
  double pitchZ;
  if(isBLayer) {
    if (m_dc1Geometry) { // Override NOVA 
      pitchZ = 300 * micrometer; 
    } else {
      pitchZ = (*pxbd)[0]->getDouble("PITCHZ") * cm;
    }
  } else {
    pitchZ = (*pxbd)[1]->getDouble("PITCHZ") * cm;
  }
  return pitchZ;
}

double OraclePixGeoManager::DesignGapRP() {
  return (*pdch)[0]->getDouble("GAPRP")*cm;
}

double OraclePixGeoManager::DesignGapZ() {
  return (*pdch)[0]->getDouble("GAPZ")*cm;
}


int OraclePixGeoManager::DesignCircuitsPerColumn() {
  //
  // This should be (*pdch)[0]->getDouble("NRPCHIP"), but in the current
  // design we prefer to have one chip in the rphi direction
  // and define the connections for the pixels in the gap
  return 1;
}
int OraclePixGeoManager::DesignCircuitsPerRow() {
  return  (*pdch)[0]->getInt("NZCHIP");
}

//
// Trapezoid parameters
//
int OraclePixGeoManager::PixelNTrap() { 
  return (*ptla)[0]->getInt("NTRAP");
}
int OraclePixGeoManager::PixelNTrapC() {
  return (*ptla)[0]->getInt("NCENT");
}
int OraclePixGeoManager::PixelNTrapE() {
  return (*ptla)[0]->getInt("NEND");
}
double OraclePixGeoManager::PixelTrapWidth() {
  return (*ptla)[0]->getDouble("WIDTH")*cm;
} 
double OraclePixGeoManager::PixelTrapHalfLength() {
  if (currentTrapType == "TMT") {
    if (currentTrap < (*ptla)[0]->getInt("NCENT")) {
      return (*pctr)[currentTrap]->getDouble("DZ")*cm;
    } else if (currentTrap < (*ptla)[0]->getInt("NEND") + (*ptla)[0]->getInt("NCENT")) {
      return (*pftr)[currentTrap-(*ptla)[0]->getInt("NCENT")]->getDouble("DZ")*cm;
    } else {
      return (*pttr)[currentTrap-(*ptla)[0]->getInt("NCENT")-(*ptla)[0]->getInt("NEND")]->getDouble("DZ")*cm;
    }
  } else if (currentTrapType == "Fluid") {
    return (*pcff)[currentTrap]->getDouble("DZ")*cm;
  } else {
    std::cout <<"Wrong Trapezoid Type ! Wrong Geometry !"<<std::endl;
    return (*pctr)[currentTrap]->getDouble("DZ")*cm;
  }
}
double OraclePixGeoManager::PixelTrapDzdr() { 
  if (currentTrapType == "TMT") {
    if (currentTrap < (*ptla)[0]->getInt("NCENT")) {
      return (*pctr)[currentTrap]->getDouble("DZDR")*deg;
    } else if (currentTrap < (*ptla)[0]->getInt("NEND") + (*ptla)[0]->getInt("NCENT")) {
      return (*pftr)[currentTrap-(*ptla)[0]->getInt("NCENT")]->getDouble("DZDR")*deg;
    } else {
      return (*pttr)[currentTrap-(*ptla)[0]->getInt("NCENT")-(*ptla)[0]->getInt("NEND")]->getDouble("DZDR")*deg;
    }
  } else if (currentTrapType == "Fluid") {
    return (*pcff)[currentTrap]->getDouble("DZDR")*deg;
  } else {
    std::cout <<"Wrong Trapezoid Type ! Wrong Geometry !"<<std::endl;
    return (*pctr)[currentTrap]->getDouble("DZDR")*deg;
  }
} 
double OraclePixGeoManager::PixelTrapHalfWidthY() {
  if (currentTrapType == "TMT") {
    if (currentTrap < (*ptla)[0]->getInt("NCENT")) {
      return (*pctr)[currentTrap]->getDouble("DY")*cm;
    } else if (currentTrap < (*ptla)[0]->getInt("NEND") + (*ptla)[0]->getInt("NCENT")) {
      return (*pftr)[currentTrap-(*ptla)[0]->getInt("NCENT")]->getDouble("DY")*cm;
    } else {
      return (*pttr)[currentTrap-(*ptla)[0]->getInt("NCENT")-(*ptla)[0]->getInt("NEND")]->getDouble("DY")*cm;
    }
  } else if (currentTrapType == "Fluid") {
    return (*pcff)[currentTrap]->getDouble("DY")*cm;
  } else {
    std::cout <<"Wrong Trapezoid Type ! Wrong Geometry !"<<std::endl;
    return (*pctr)[currentTrap]->getDouble("DY")*cm;
  }
} 
double OraclePixGeoManager::PixelTrapHalfWidthXzn() { 
  if (currentTrapType == "TMT") {
    if (currentTrap < (*ptla)[0]->getInt("NCENT")) {
      return (*pctr)[currentTrap]->getDouble("DX1")*cm;
    } else if (currentTrap < (*ptla)[0]->getInt("NEND") + (*ptla)[0]->getInt("NCENT")) {
      return (*pftr)[currentTrap-(*ptla)[0]->getInt("NCENT")]->getDouble("DX1")*cm;
    } else {
      return (*pttr)[currentTrap-(*ptla)[0]->getInt("NCENT")-(*ptla)[0]->getInt("NEND")]->getDouble("DX1")*cm;
    }
  } else if (currentTrapType == "Fluid") {
    return (*pcff)[currentTrap]->getDouble("DX1")*cm;
  } else {
    std::cout <<"Wrong Trapezoid Type ! Wrong Geometry !"<<std::endl;
    return (*pctr)[currentTrap]->getDouble("DX1")*cm;
  }
}
double OraclePixGeoManager::PixelTrapHalfWidthXzp() {
  if (currentTrapType == "TMT") {
    if (currentTrap < (*ptla)[0]->getInt("NCENT")) {
      return (*pctr)[currentTrap]->getDouble("DX2")*cm;
    } else if (currentTrap < (*ptla)[0]->getInt("NEND") + (*ptla)[0]->getInt("NCENT")) {
      return (*pftr)[currentTrap-(*ptla)[0]->getInt("NCENT")]->getDouble("DX2")*cm;
    } else {
      return (*pttr)[currentTrap-(*ptla)[0]->getInt("NCENT")-(*ptla)[0]->getInt("NEND")]->getDouble("DX2")*cm;
    }
  } else if (currentTrapType == "Fluid") {
    return (*pcff)[currentTrap]->getDouble("DX2")*cm;
  } else {
    std::cout <<"Wrong Trapezoid Type ! Wrong Geometry !"<<std::endl;
    return (*pctr)[currentTrap]->getDouble("DX2")*cm;
  }
} 
double OraclePixGeoManager::PixelTrapPosx() {
  if (currentTrapType == "TMT") {
    if (currentTrap < (*ptla)[0]->getInt("NCENT")) {
      return (*pctr)[currentTrap]->getDouble("X")*cm;
    } else if (currentTrap < (*ptla)[0]->getInt("NEND") + (*ptla)[0]->getInt("NCENT")) {
      return (*pftr)[currentTrap-(*ptla)[0]->getInt("NCENT")]->getDouble("X")*cm;
    } else {
      return (*pttr)[currentTrap-(*ptla)[0]->getInt("NCENT")-(*ptla)[0]->getInt("NEND")]->getDouble("X")*cm;
    }
  } else if (currentTrapType == "Fluid") {
    //JBdV temporary
    //return (*pcff)[currentTrap]->getDouble("X")*cm;
    double x = (*pcff)[currentTrap]->getDouble("X")*cm;
    return x + 0.05;
  } else {
    std::cout <<"Wrong Trapezoid Type ! Wrong Geometry !"<<std::endl;
    return (*pctr)[currentTrap]->getDouble("X")*cm;
  }
} 
double OraclePixGeoManager::PixelTrapPosy() {
  if (currentTrapType == "TMT") {
    if (currentTrap < (*ptla)[0]->getInt("NCENT")) {
      return (*pctr)[currentTrap]->getDouble("Y")*cm;
    } else if (currentTrap < (*ptla)[0]->getInt("NEND") + (*ptla)[0]->getInt("NCENT")) {
      return (*pftr)[currentTrap-(*ptla)[0]->getInt("NCENT")]->getDouble("Y")*cm;
    } else {
      return (*pttr)[currentTrap-(*ptla)[0]->getInt("NCENT")-(*ptla)[0]->getInt("NEND")]->getDouble("Y")*cm;
    }
  } else if (currentTrapType == "Fluid") {
    return (*pcff)[currentTrap]->getDouble("Y")*cm;
  } else {
    std::cout <<"Wrong Trapezoid Type ! Wrong Geometry !"<<std::endl;
    return (*pctr)[currentTrap]->getDouble("Y")*cm;
  }
}
double OraclePixGeoManager::PixelTrapPosz() { 
  if (currentTrapType == "TMT") {
    if (currentTrap < (*ptla)[0]->getInt("NCENT")) {
      return (*pctr)[currentTrap]->getDouble("Z")*cm;
    } else if (currentTrap < (*ptla)[0]->getInt("NEND") + (*ptla)[0]->getInt("NCENT")) {
      return (*pftr)[currentTrap-(*ptla)[0]->getInt("NCENT")]->getDouble("Z")*cm;
    } else {
      return (*pttr)[currentTrap-(*ptla)[0]->getInt("NCENT")-(*ptla)[0]->getInt("NEND")]->getDouble("Z")*cm;
    }
  } else if (currentTrapType == "Fluid") {
    return (*pcff)[currentTrap]->getDouble("Z")*cm;
  } else {
    std::cout <<"Wrong Trapezoid Type ! Wrong Geometry !"<<std::endl;
    return (*pctr)[currentTrap]->getDouble("Z")*cm;
  }
} 
int OraclePixGeoManager::PixelTrapLabel() { 
  if (currentTrapType == "TMT") {
    if (currentTrap < (*ptla)[0]->getInt("NCENT")) {
      return (int)(*pctr)[currentTrap]->getDouble("ITRD");
    } else if (currentTrap < (*ptla)[0]->getInt("NEND") + (*ptla)[0]->getInt("NCENT")) {
      return (int)(*pftr)[currentTrap-(*ptla)[0]->getInt("NCENT")]->getDouble("ITRD");
    } else {
      return (int)(*pttr)[currentTrap-(*ptla)[0]->getInt("NCENT")-(*ptla)[0]->getInt("NEND")]->getDouble("ITRD");
    }
  } else if (currentTrapType == "Fluid") {
    return (int)(*pcff)[currentTrap]->getDouble("ITRD")*cm;
  } else {
    std::cout <<"Wrong Trapezoid Type ! Wrong Geometry !"<<std::endl;
    return (int)(*pctr)[currentTrap]->getDouble("ITRD");
  }
} 
int OraclePixGeoManager::PixelTrapMat() { 
  return (int)(*pcff)[currentTrap]->getDouble("MAT");
}

//
// Omega and Tube parameters
//
int OraclePixGeoManager::PixelOmegaNParaEl() {
  return (*pome)[0]->getInt("NPOSI");
}
int OraclePixGeoManager::PixelOmegaNCyliEl() {
  return (*pome)[0]->getInt("NPOTI");  
}
int OraclePixGeoManager::PixelConnectorANParaEl() {
  return (*pome)[0]->getInt("NPOAI");  
}
int OraclePixGeoManager::PixelConnectorCNParaEl() {
  return (*pome)[0]->getInt("NPOCI");  
}
int OraclePixGeoManager::PixelConnectorBNCyliEl() {
  return (*pome)[0]->getInt("NPOBI");  
}


// Cylindrical elements
double OraclePixGeoManager::PixelPotiHalfLength(std::string cyliType) {
  if (cyliType == "Poti") {
    return (*poti)[currentCyliEl]->getDouble("DZ")*cm;
  } else if (cyliType == "Pobi") {
    return (*pobi)[currentCyliEl]->getDouble("DZ")*cm;
  } else {
    std::cout <<"Wrong Cylindrical element Type ! Wrong Geometry !"<<std::endl;
    return (*poti)[currentCyliEl]->getDouble("DZ")*cm;
  }
}
double OraclePixGeoManager::PixelPotiRmin(std::string cyliType) {
  if (cyliType == "Poti") {
    return (*poti)[currentCyliEl]->getDouble("RMIN")*cm;
  } else if (cyliType == "Pobi") {
    return (*pobi)[currentCyliEl]->getDouble("RMIN")*cm;
  } else {
    std::cout <<"Wrong Cylindrical element Type ! Wrong Geometry !"<<std::endl;
    return (*poti)[currentCyliEl]->getDouble("RMIN")*cm;
  }
}
double OraclePixGeoManager::PixelPotiRmax(std::string cyliType) {
  if (cyliType == "Poti") {
    return (*poti)[currentCyliEl]->getDouble("RMAX")*cm;
  } else if (cyliType == "Pobi") {
    return (*pobi)[currentCyliEl]->getDouble("RMAX")*cm;
  } else {
    std::cout <<"Wrong Cylindrical element Type ! Wrong Geometry !"<<std::endl;
    return (*poti)[currentCyliEl]->getDouble("RMAX")*cm;
  }
}
double OraclePixGeoManager::PixelPotiPhi1(std::string cyliType) {
  if (cyliType == "Poti") {
    return (*poti)[currentCyliEl]->getDouble("PHI1")*deg;
  } else if (cyliType == "Pobi") {
    return (*pobi)[currentCyliEl]->getDouble("PHI1")*deg;
  } else {
    std::cout <<"Wrong Cylindrical element Type ! Wrong Geometry !"<<std::endl;
    return (*poti)[currentCyliEl]->getDouble("PHI1")*deg;
  }
}
double OraclePixGeoManager::PixelPotiPhi2(std::string cyliType) {
  if (cyliType == "Poti") {
    return (*poti)[currentCyliEl]->getDouble("PHI2")*deg;
  } else if (cyliType == "Pobi") {
    return (*pobi)[currentCyliEl]->getDouble("PHI2")*deg;
  } else {
    std::cout <<"Wrong Cylindrical element Type ! Wrong Geometry !"<<std::endl;
    return (*poti)[currentCyliEl]->getDouble("PHI2")*deg;
  }
}
double OraclePixGeoManager::PixelPotiPosx(std::string cyliType) {
  if (cyliType == "Poti") {
    //JBdV temporary
    //return (*poti)[currentCyliEl]->getDouble("X")*cm;
    double x = (*poti)[currentCyliEl]->getDouble("X")*cm;
    if (currentCyliEl < 3) return x + 0.1;
    return x + 0.05;
  } else if (cyliType == "Pobi") {
    return (*pobi)[currentCyliEl]->getDouble("X")*cm;
  } else {
    std::cout <<"Wrong Cylindrical element Type ! Wrong Geometry !"<<std::endl;
    return (*poti)[currentCyliEl]->getDouble("X")*cm;
  }
}
double OraclePixGeoManager::PixelPotiPosy(std::string cyliType) {
  if (cyliType == "Poti") {
    return (*poti)[currentCyliEl]->getDouble("Y")*cm;
  } else if (cyliType == "Pobi") {
    return (*pobi)[currentCyliEl]->getDouble("Y")*cm;
  } else {
    std::cout <<"Wrong Cylindrical element Type ! Wrong Geometry !"<<std::endl;
    return (*poti)[currentCyliEl]->getDouble("Y")*cm;
  }
}
double OraclePixGeoManager::PixelPotiPosz(std::string cyliType) {
  if (cyliType == "Poti") {
    return (*poti)[currentCyliEl]->getDouble("Z")*cm;
  } else if (cyliType == "Pobi") {
    return (*pobi)[currentCyliEl]->getDouble("Z")*cm;
  } else {
    std::cout <<"Wrong Cylindrical element Type ! Wrong Geometry !"<<std::endl;
    return (*poti)[currentCyliEl]->getDouble("Z")*cm;
  }
}
int OraclePixGeoManager::PixelPotiLabel(std::string cyliType) {
  if (cyliType == "Poti") {
    return (int)(*poti)[currentCyliEl]->getDouble("ICYL");
  } else if (cyliType == "Pobi") {
    return (int)(*pobi)[currentCyliEl]->getDouble("ICYL");
  } else {
    std::cout <<"Wrong Cylindrical element Type ! Wrong Geometry !"<<std::endl;
    return (int)(*poti)[currentCyliEl]->getDouble("ICYL");
  }
}
int OraclePixGeoManager::PixelMaterialCyli(std::string cyliType) {
  if (cyliType == "Poti") {
    int imat = (int)(*poti)[currentCyliEl]->getDouble("MAT");
    if (imat == 3) {
      if (currentLD < 2) {
	return imat;
      } else {
	return 7;
      }
    }
    return (int)(*poti)[currentCyliEl]->getDouble("MAT");
  } else if (cyliType == "Pobi") {
    return (int)(*pobi)[currentCyliEl]->getDouble("MAT");
  } else {
    std::cout <<"Wrong Cylindrical element Type ! Wrong Geometry !"<<std::endl;
    return (int)(*poti)[currentCyliEl]->getDouble("MAT");
  }
}

// Parallelepipedic elements
double OraclePixGeoManager::PixelPosiHalfLength() {
  if (currentParaElType == "Posi") {
    return (*posi)[currentParaEl]->getDouble("DZ")*cm;
  } else if (currentParaElType == "Poai") {
    return (*poai)[currentParaEl]->getDouble("DZ")*cm;
  } else if (currentParaElType == "Poci") {
    return (*poci)[currentParaEl]->getDouble("DZ")*cm;
  } else {
    std::cout <<"Wrong Parallelepipedic element Type ! Wrong Geometry ! (PixelPosiHalfLength)"<<std::endl;
    return (*posi)[currentParaEl]->getDouble("DZ")*cm;
  }
}
double OraclePixGeoManager::PixelPosiWidth() {
  if (currentParaElType == "Posi") {
    return (*posi)[currentParaEl]->getDouble("DY")*cm;
  } else if (currentParaElType == "Poai") {
    return (*poai)[currentParaEl]->getDouble("DY")*cm;
  } else if (currentParaElType == "Poci") {
    return (*poci)[currentParaEl]->getDouble("DY")*cm;
  } else {
    std::cout <<"Wrong Parallelepipedic element Type ! Wrong Geometry ! (PixelPosiWidth)"<<std::endl;
    return (*posi)[currentParaEl]->getDouble("DY")*cm;
  }
}
double OraclePixGeoManager::PixelPosiThick() {
  if (currentParaElType == "Posi") {
    //JBdV temporary
    if (currentParaEl == 2 || currentParaEl == 3) return 0.1;
    if (currentParaEl == 4) return 0.05;
    //
    return (*posi)[currentParaEl]->getDouble("DX")*cm;
  } else if (currentParaElType == "Poai") {
    return (*poai)[currentParaEl]->getDouble("DX")*cm;
  } else if (currentParaElType == "Poci") {
    return (*poci)[currentParaEl]->getDouble("DX")*cm;
  } else {
    std::cout <<"Wrong Parallelepipedic element Type ! Wrong Geometry ! (PixelPosiThick)"<<std::endl;
    return (*posi)[currentParaEl]->getDouble("DX")*cm;
  }
}
double OraclePixGeoManager::PixelPosiTilt() {
  if (currentParaElType == "Posi") {
    return (*posi)[currentParaEl]->getDouble("TILT")*deg;
  } else if (currentParaElType == "Poai") {
    return (*poai)[currentParaEl]->getDouble("TILT")*deg;
  } else if (currentParaElType == "Poci") {
    return (*poci)[currentParaEl]->getDouble("TILT")*deg;
  } else {
    std::cout <<"Wrong Parallelepipedic element Type ! Wrong Geometry ! (PixelPosiTilt)"<<std::endl;
    return (*posi)[currentParaEl]->getDouble("TILT")*deg;
  }
}
double OraclePixGeoManager::PixelPosiPosx() {
  if (currentParaElType == "Posi") {
    //JBdV temporary
    //return (*posi)[currentParaEl]->getDouble("X")*cm;
    double x = (*posi)[currentParaEl]->getDouble("X")*cm;
    if (currentParaEl < 2 || currentParaEl == 5 || currentParaEl == 6) return x + 0.1;
    if (currentParaEl > 6) return x + 0.05;
    if (currentParaEl == 2 || currentParaEl == 3) return x + 0.05;
    return x + 0.025;
  } else if (currentParaElType == "Poai") {
    return (*poai)[currentParaEl]->getDouble("X")*cm;
  } else if (currentParaElType == "Poci") {
    return (*poci)[currentParaEl]->getDouble("X")*cm;
  } else {
    std::cout <<"Wrong Parallelepipedic element Type ! Wrong Geometry ! (PixelPosiPosx)"<<std::endl;
    return (*posi)[currentParaEl]->getDouble("X")*cm;
  }
}
double OraclePixGeoManager::PixelPosiPosy() {
  if (currentParaElType == "Posi") {
    return (*posi)[currentParaEl]->getDouble("Y")*cm;
  } else if (currentParaElType == "Poai") {
    return (*poai)[currentParaEl]->getDouble("Y")*cm;
  } else if (currentParaElType == "Poci") {
    return (*poci)[currentParaEl]->getDouble("Y")*cm;
  } else {
    std::cout <<"Wrong Parallelepipedic element Type ! Wrong Geometry ! (PixelPosiPosy)"<<std::endl;
    return (*posi)[currentParaEl]->getDouble("Y")*cm;
  }  
}
double OraclePixGeoManager::PixelPosiPosz() {
  if (currentParaElType == "Posi") {
    return (*posi)[currentParaEl]->getDouble("Z")*cm;
  } else if (currentParaElType == "Poai") {
    return (*poai)[currentParaEl]->getDouble("Z")*cm;
  } else if (currentParaElType == "Poci") {
    return (*poci)[currentParaEl]->getDouble("Z")*cm;
  } else {
    std::cout <<"Wrong Parallelepipedic element Type ! Wrong Geometry ! (PixelPosiPosz)"<<std::endl;
    return (*posi)[currentParaEl]->getDouble("Z")*cm;
  }  
}
double OraclePixGeoManager::PixelPosiRotz() {
  if (currentParaElType == "Posi") {
    return (*posi)[currentParaEl]->getDouble("ALPHAZ")*deg;
  } else if (currentParaElType == "Poai") {
    return (*poai)[currentParaEl]->getDouble("ALPHAZ")*deg;
  } else if (currentParaElType == "Poci") {
    return (*poci)[currentParaEl]->getDouble("ALPHAZ")*deg;
  } else {
    std::cout <<"Wrong Parallelepipedic element Type ! Wrong Geometry ! (PixelPosiRotz)"<<std::endl;
    return (*posi)[currentParaEl]->getDouble("ALPHAZ")*deg;
  }
}
int OraclePixGeoManager::PixelPosiLabel() {
  if (currentParaElType == "Posi") {
    return (int)(*posi)[currentParaEl]->getDouble("IBOX");
  } else if (currentParaElType == "Poai") {
    return (int)(*poai)[currentParaEl]->getDouble("IBOX");
  } else if (currentParaElType == "Poci") {
    return (int)(*poci)[currentParaEl]->getDouble("IBOX");
  } else {
    std::cout <<"Wrong Parallelepipedic element Type ! Wrong Geometry ! (PixelPosiLabel)"<<std::endl;
    return (int)(*posi)[currentParaEl]->getDouble("IBOX");
  }
}
int OraclePixGeoManager::PixelMaterialPara() {
  if (currentParaElType == "Posi") {
    int imat = (int)(*posi)[currentParaEl]->getDouble("MAT");
    if (imat == 3) {
      if (currentLD < 2) {
	return imat;
      } else {
	return 7;
      }
    }
    return (int)(*posi)[currentParaEl]->getDouble("MAT");
  } else if (currentParaElType == "Poai") {
    return (int)(*poai)[currentParaEl]->getDouble("MAT");
  } else if (currentParaElType == "Poci") {
    return (int)(*poci)[currentParaEl]->getDouble("MAT");
  } else {
    std::cout <<"Wrong Parallelepipedic element Type ! Wrong Material !"<<std::endl;
    return (int)(*posi)[currentParaEl]->getDouble("MAT");
  }
}

//Fluid
int OraclePixGeoManager::PixelFluidOrient() {
  std::string var = "ORIENT_";
  std::ostringstream o;
  o << m_phi;
  var = var+o.str();
  return (int)(*pccf)[currentLD]->getInt(var);
}



#endif
