#include "PixelGeoModel/GeoPixelBarrel.h"
#include "PixelGeoModel/GeoPixelLayer.h"
#include "PixelGeoModel/GeoPixelServices.h"
#include "GeoModelKernel/GeoTube.h"
#include "GeoModelKernel/GeoLogVol.h"
#include "GeoModelKernel/GeoNameTag.h"
#include "GeoModelKernel/GeoIdentifierTag.h"
#include "GeoModelKernel/GeoPhysVol.h"
#include "GeoModelKernel/GeoMaterial.h"
#include "GeoModelKernel/GeoAlignableTransform.h"
#include "InDetReadoutGeometry/PixelDetectorManager.h"
#include "Identifier/Identifier.h"

// For the Msg Svc
#include "AthenaKernel/getMessageSvc.h"
#include "GaudiKernel/MsgStream.h"


using namespace std;

GeoVPhysVol* GeoPixelBarrel::Build( ) {

  MsgStream log(Athena::getMessageSvc(), "PixelDetectorFactory");

  //
  // create the Barrel Mother volume
  // 
  double rmin = gmt_mgr->PixelBarrelRMin();
  double rmax = gmt_mgr->PixelBarrelRMax();
  double halflength = gmt_mgr->PixelBarrelHalfLength();
  GeoMaterial* air = mat_mgr->getMaterial("std::Air");
  const GeoTube* barrelTube = new GeoTube(rmin,rmax,halflength);
  const GeoLogVol* barrelLog = new GeoLogVol("barrelLog",barrelTube,air);
  GeoPhysVol* barrelPhys = new GeoPhysVol(barrelLog);
  //
  // Build the layers inside
  //
  GeoPixelLayer layer;
  string lname[3];
  lname[0] = "InnerLayer";
  lname[1] = "CenterLayer";
  lname[2] = "OuterLayer"; 
  for(int ii = 0; ii < gmt_mgr->PixelBarrelNLayer(); ii++) {
    //cout << "Layer" << ii << endl;
    gmt_mgr->SetCurrentLD(ii);
    if(gmt_mgr->isLDPresent()) {
      GeoAlignableTransform * xform = new GeoAlignableTransform(HepTransform3D()); 
      GeoVPhysVol* layerphys = layer.Build();
      GeoNameTag *tag = new GeoNameTag(lname[ii]);         
      barrelPhys->add(tag);
      barrelPhys->add(new GeoIdentifierTag(ii));
      barrelPhys->add(xform);
      barrelPhys->add(layerphys);
      
      // Store the transform (at level 1)
      Identifier id = DDmgr->getIdHelper()->wafer_id(0,ii,0,0);
      DDmgr->addAlignableTransform(1, id, xform, layerphys);
    } else {
      log << MSG::DEBUG << "Layer " << ii << " not built" << endreq;
    }
  }
  if(gmt_mgr->DoServices() ) {
    //
    // Add the services inside the barrel volume
    //
    GeoPixelServices brlsvc = GeoPixelServices("/Pixel/PixelBarrel");
    for(int ii =0; ii< brlsvc.NCylinders(); ii++) {
      brlsvc.SetCylinder(ii);
      GeoNameTag* tag = new GeoNameTag("Outside Barrel Service"); // pourquoi "Oustside" et pas "Inside"?
      Hep3Vector pos(0.,0.,brlsvc.ZPos() );
      GeoTransform* xform = new GeoTransform(HepTransform3D(HepRotation(),pos));
      barrelPhys->add(tag);
      barrelPhys->add(xform);
      barrelPhys->add(brlsvc.Build() );
    }
  }
  return barrelPhys;
}
