#include "PixelGeoModel/GeoPixelBcm.h"
#include "PixelGeoModel/GeoPixelBcmWall.h"
#include "PixelGeoModel/GeoPixelLayer.h"
#include "PixelGeoModel/GeoPixelServices.h"
#include "GeoModelKernel/GeoTube.h"
#include "GeoModelKernel/GeoBox.h"
#include "GeoModelKernel/GeoLogVol.h"
#include "GeoModelKernel/GeoNameTag.h"
#include "GeoModelKernel/GeoIdentifierTag.h"
#include "GeoModelKernel/GeoPhysVol.h"
#include "GeoModelKernel/GeoMaterial.h"
#include "GeoModelKernel/GeoAlignableTransform.h"
#include "InDetReadoutGeometry/PixelDetectorManager.h"
#include "Identifier/Identifier.h"

// For the Msg Svc
#include "AthenaKernel/getMessageSvc.h"
#include "GaudiKernel/MsgStream.h"


using namespace std;

GeoVPhysVol* GeoPixelBcm::Build( ) {

  MsgStream log(Athena::getMessageSvc(), "PixelDetectorFactory");

  //
  // create the Barrel Mother volume
  // 
//  double rmin = gmt_mgr->PixelBarrelRMin();
//  double rmax = gmt_mgr->PixelBarrelRMax();
//  double halflength = gmt_mgr->PixelBarrelHalfLength();

//  double r=5.639;
//  double z=183.8 + 0.185;

  double modLength=132.969;
  double modWidth=33.02;
  double modHeight=21.717;

  GeoMaterial* air=mat_mgr->getMaterial("std::Air");

  const GeoBox* bcmModBox = new GeoBox(modHeight/2,modWidth/2,modLength/2);
  const GeoLogVol* bcmModLog=new GeoLogVol("bcmModLog",bcmModBox,air);
  GeoPhysVol* bcmModPhys = new GeoPhysVol(bcmModLog);


  double wallThickness=0.435;
  double pcbThickness=1.435;
  double reducedWidth = modWidth - 2*wallThickness;


		//add walls
		//GeoMaterial* copper = mat_mgr->getMaterial("std::Copper");

		GeoPixelBcmWall wall;

		//top wall	

		//const GeoBox* bcmTopWallBox = new GeoBox(wallThickness/2,reducedWidth/2,modLength/2);
		//const GeoLogVol* bcmTopWallLog = new GeoLogVol("bcmTopWallLog",bcmTopWallBox,copper);
		GeoVPhysVol* bcmTopWallPhys = wall.Build(reducedWidth/2,modLength/2);
		
		double x,y,z;
		x = modHeight/2 - wallThickness/2;
		y = 0;
		z = 0;

		Hep3Vector topWallPos(x,y,z);
		HepRotation rm;
		GeoTransform* xform = new GeoTransform(HepTransform3D(rm,topWallPos));
		GeoNameTag* tag = new GeoNameTag("BCM Module Top Wall");
		bcmModPhys->add(tag);
		bcmModPhys->add(xform);
		bcmModPhys->add(bcmTopWallPhys);

		//front wall
		double frontWallHeight=7.747-wallThickness-pcbThickness;
		//const GeoBox* bcmFrontWallBox = new GeoBox(frontWallHeight/2,reducedWidth/2,wallThickness/2);
		//const GeoLogVol* bcmFrontWallLog = new GeoLogVol("bcmFrontWallLog",bcmFrontWallBox,copper);
		GeoVPhysVol* bcmFrontWallPhys = wall.Build(reducedWidth/2,frontWallHeight/2);
		
		x = modHeight/2 - wallThickness - (7.747-wallThickness-pcbThickness)/2;
		y = 0;
		z = -(modLength/2 -wallThickness/2);

		Hep3Vector frontWallPos(x,y,z);
		HepRotation rmFront;
		rmFront.rotateY(90.*deg);
		xform = new GeoTransform(HepTransform3D(rmFront,frontWallPos));
		tag = new GeoNameTag("BCM Module Front Wall");
		bcmModPhys->add(tag);
		bcmModPhys->add(xform);
		bcmModPhys->add(bcmFrontWallPhys);


		//back wall
		double backWallHeight=modHeight-2*wallThickness;
		//const GeoBox* bcmBackWallBox = new GeoBox(backWallHeight/2,reducedWidth/2,wallThickness/2);
		//const GeoLogVol* bcmBackWallLog = new GeoLogVol("bcmBackWallLog",bcmBackWallBox,copper);
		GeoVPhysVol* bcmBackWallPhys = wall.Build(reducedWidth/2,backWallHeight/2);
		
		x = 0;
		y = 0;
		z = modLength/2 -wallThickness/2;

		Hep3Vector backWallPos(x,y,z);
		HepRotation rmBack;
		rmBack.rotateY(-90.*deg);
		xform = new GeoTransform(HepTransform3D(rmBack,backWallPos));
		tag = new GeoNameTag("BCM Module Back Wall");
		bcmModPhys->add(tag);
		bcmModPhys->add(xform);
		bcmModPhys->add(bcmBackWallPhys);


		//lower front wall
		double lowerFrontWallHeight=13.97 - wallThickness;
		//const GeoBox* bcmLowerFrontWallBox = new GeoBox(lowerFrontWallHeight/2,reducedWidth/2,wallThickness/2);
		//const GeoLogVol* bcmLowerFrontWallLog = new GeoLogVol("bcmLowerFrontWallLog",bcmLowerFrontWallBox,copper);
		GeoVPhysVol* bcmLowerFrontWallPhys = wall.Build(reducedWidth/2,lowerFrontWallHeight/2);
		
		x = -(modHeight/2 - wallThickness - lowerFrontWallHeight/2);
		y = 0;
		z = modLength/2 - 49.657 + wallThickness/2;

		Hep3Vector lowerFrontWallPos(x,y,z);
		//HepRotation rm;
		xform = new GeoTransform(HepTransform3D(rmFront,lowerFrontWallPos));
		tag = new GeoNameTag("BCM Module Lower Front Wall");
		bcmModPhys->add(tag);
		bcmModPhys->add(xform);
		bcmModPhys->add(bcmLowerFrontWallPhys);



		//bottom wall
		double bottomWallLength=49.657;
		//const GeoBox* bcmBottomWallBox = new GeoBox(wallThickness/2,reducedWidth/2,bottomWallLength/2);
		//const GeoLogVol* bcmBottomWallLog = new GeoLogVol("bcmBottomWallLog",bcmBottomWallBox,copper);
		GeoVPhysVol* bcmBottomWallPhys = wall.Build(reducedWidth/2,bottomWallLength/2);
		
		x = -(modHeight/2 - wallThickness/2);
		y = 0;
		z = modLength/2 - bottomWallLength/2;

		Hep3Vector bottomWallPos(x,y,z);
		HepRotation rmBottom;
		rmBottom.rotateZ(180.*deg);
		xform = new GeoTransform(HepTransform3D(rmBottom,bottomWallPos));
		tag = new GeoNameTag("BCM Module Back Wall");
		bcmModPhys->add(tag);
		bcmModPhys->add(xform);
		bcmModPhys->add(bcmBottomWallPhys);


		//side walls
		//rear
		double rearSideWallLength=49.657;
		//const GeoBox* bcmRearSideWall1Box = new GeoBox(modHeight/2,wallThickness/2,rearSideWallLength/2);
		//const GeoLogVol* bcmRearSideWall1Log = new GeoLogVol("bcmRearSideWall1Log",bcmRearSideWall1Box,copper);
		GeoVPhysVol* bcmRearSideWall1Phys = wall.Build(modHeight/2,rearSideWallLength/2);
		
		x = 0;
		y = modWidth/2 - wallThickness/2;
		z = modLength/2 - rearSideWallLength/2;

		Hep3Vector rearSideWall1Pos(x,y,z);
		HepRotation rmSide1;
		rmSide1.rotateZ(90.*deg);
		xform = new GeoTransform(HepTransform3D(rmSide1,rearSideWall1Pos));
		tag = new GeoNameTag("BCM Module Rear Side Wall 1");
		bcmModPhys->add(tag);
		bcmModPhys->add(xform);
		bcmModPhys->add(bcmRearSideWall1Phys);


		//const GeoBox* bcmRearSideWall2Box = new GeoBox(modHeight/2,wallThickness/2,rearSideWallLength/2);
		//const GeoLogVol* bcmRearSideWall2Log = new GeoLogVol("bcmRearSideWall2Log",bcmRearSideWall2Box,copper);
		GeoVPhysVol* bcmRearSideWall2Phys = wall.Build(modHeight/2,rearSideWallLength/2);
		
		x = 0;
		y = -(modWidth/2 - wallThickness/2);
		z = modLength/2 - rearSideWallLength/2;

		Hep3Vector rearSideWall2Pos(x,y,z);
		HepRotation rmSide2;
		rmSide2.rotateZ(-90.*deg);
		xform = new GeoTransform(HepTransform3D(rmSide2,rearSideWall2Pos));
		tag = new GeoNameTag("BCM Module Rear Side Wall 2");
		bcmModPhys->add(tag);
		bcmModPhys->add(xform);
		bcmModPhys->add(bcmRearSideWall2Phys);
		
		//front
		double frontSideWallLength = modLength - 49.657;
		double frontSideWallHeight = 7.747;
		//const GeoBox* bcmFrontSideWall1Box = new GeoBox(frontSideWallHeight/2,wallThickness/2,frontSideWallLength/2);
		//const GeoLogVol* bcmFrontSideWall1Log = new GeoLogVol("bcmFrontSideWall1Log",bcmFrontSideWall1Box,copper);
		GeoVPhysVol* bcmFrontSideWall1Phys = wall.Build(frontSideWallHeight/2,frontSideWallLength/2);
		
		x = modHeight/2 - frontSideWallHeight/2;
		y = modWidth/2 - wallThickness/2;
		z = -(modLength/2 - frontSideWallLength/2);

		Hep3Vector frontSideWall1Pos(x,y,z);
		//HepRotation rm;
		xform = new GeoTransform(HepTransform3D(rmSide1,frontSideWall1Pos));
		tag = new GeoNameTag("BCM Module Front Side Wall 1");
		bcmModPhys->add(tag);
		bcmModPhys->add(xform);
		bcmModPhys->add(bcmFrontSideWall1Phys);

		//const GeoBox* bcmFrontSideWall2Box = new GeoBox(frontSideWallHeight/2,wallThickness/2,frontSideWallLength/2);
		//const GeoLogVol* bcmFrontSideWall2Log = new GeoLogVol("bcmFrontSideWall2Log",bcmFrontSideWall2Box,copper);
		GeoVPhysVol* bcmFrontSideWall2Phys = wall.Build(frontSideWallHeight/2,frontSideWallLength/2);
		
		x = modHeight/2 - frontSideWallHeight/2;
		y = -(modWidth/2 - wallThickness/2);
		z = -(modLength/2 - frontSideWallLength/2);

		Hep3Vector frontSideWall2Pos(x,y,z);
		//HepRotation rm;
		xform = new GeoTransform(HepTransform3D(rmSide2,frontSideWall2Pos));
		tag = new GeoNameTag("BCM Module Front Side Wall 2");
		bcmModPhys->add(tag);
		bcmModPhys->add(xform);
		bcmModPhys->add(bcmFrontSideWall2Phys);





		//add pcb
		double pcbLength=modLength-wallThickness;
		double pcbWidth=modWidth-2*wallThickness;
		

		//GeoMaterial* g10 = mat_mgr->getMaterial("std::G10");

		//const GeoBox* bcmPcbBox = new GeoBox(pcbThickness/2,pcbWidth/2,pcbLength/2);
  		//const GeoLogVol* bcmPcbLog=new GeoLogVol("bcmPcbLog",bcmPcbBox,g10);
		GeoVPhysVol* bcmPcbPhys = wall.Build(pcbWidth/2,pcbLength/2,1.4,0.035);
		

		double pcbX=modHeight/2 - 7.747 + pcbThickness/2; 	//3.8615
		double pcbZ=-wallThickness/2;
		Hep3Vector pcbPos(pcbX,0,pcbZ);
		HepRotation rmPcb;
		GeoTransform* xformPcb = new GeoTransform(HepTransform3D(rmPcb,pcbPos));
		tag = new GeoNameTag("BCM PCB");
		bcmModPhys->add(tag);
  		bcmModPhys->add(xformPcb);
		bcmModPhys->add(bcmPcbPhys );		


		//add diamond
		
  		double diamondThickness=1.25;
  		double diamondWidth = 10.0;

 		 GeoMaterial* diamond = mat_mgr->getMaterial("std::Carbon");

 	/*	GeoMaterial* diamond=new GeoMaterial("diamond",3.52*gram/cm3);
  		GeoElement* carbon = new GeoElement("Carbon","C",6.0,12.0112);
  		diamond->add(carbon,1.0);
  		diamond->lock();	*/

 		 const GeoBox* bcmDiamondBox = new GeoBox(diamondThickness/2,diamondWidth/2,diamondWidth/2);
  		const GeoLogVol* bcmDiamondLog = new GeoLogVol("bcmDiamondLog",bcmDiamondBox,diamond);
 		 GeoPhysVol* bcmDiamondPhys = new GeoPhysVol(bcmDiamondLog);
 


		x = modHeight/2 - 7.747 + pcbThickness + 0.39 + diamondThickness/2;   //4.1265
		z = -59.7845;
    		Hep3Vector posD(x,0,z);
		HepRotation rmD;
    		GeoTransform* xformD = new GeoTransform(HepTransform3D(rmD,posD));
		
  		tag = new GeoNameTag("BCM Diamond");
		bcmModPhys->add(tag);
		//envelopePhys->add(new GeoIdentifierTag(k));
  		bcmModPhys->add(xformD);
		bcmModPhys->add(bcmDiamondPhys );		












  return bcmModPhys;
}
