//
// This class builds one Chip to be placed in one module
//
// The information on which layer to build is retrieved by the Geometry Manager
//
// Contained by: PixelModule_log
//
// Contains: nothing
//
#include "PixelGeoModel/GeoPixelChip.h"
#include "GeoModelKernel/GeoBox.h"
#include "GeoModelKernel/GeoLogVol.h"
#include "GeoModelKernel/GeoNameTag.h"
#include "GeoModelKernel/GeoPhysVol.h"
#include "GeoModelKernel/GeoMaterial.h"

GeoVPhysVol* GeoPixelChip::Build() {
  //
  // Dimensions
  //
  double length = gmt_mgr->PixelChipLength();
  double thickness = gmt_mgr->PixelChipThickness();
  double width = gmt_mgr->PixelChipWidth();
  GeoMaterial* chipMat = mat_mgr->getMaterial("pix::Chip");
  std::string LogName = "chipBrlLog";
  //
  // In G3 is like this...
  //
  if(gmt_mgr->isEndcap() ) {
    chipMat =  mat_mgr->getMaterial("std::Silicon");
    LogName = "chipECLog";
  }
  const GeoBox* chipBox = new GeoBox(thickness/2.,width/2.,length/2.);
  GeoLogVol* theChip = new GeoLogVol(LogName,chipBox,chipMat);
  GeoPhysVol* chipPhys = new GeoPhysVol(theChip);
  //
  // Add its weight to the full detector
  //
  double weichip = chipBox->volume()*chipMat->getDensity()/gram;
  //  std::cout<<"And the Chip Weight is "<<weichip<<" grams"<<std::endl;
  gmt_mgr->AddWeight(weichip);
  gmt_mgr->AddWeightModule(weichip);
  //
  return chipPhys;
}
