#include "PixelGeoModel/GeoPixelEndCap.h"
#include "PixelGeoModel/GeoPixelDisk.h"
#include "PixelGeoModel/GeoPixelECCable.h"
#include "PixelGeoModel/GeoPixelServices.h"
#include "GeoModelKernel/GeoTube.h"
#include "GeoModelKernel/GeoLogVol.h"
#include "GeoModelKernel/GeoNameTag.h"
#include "GeoModelKernel/GeoIdentifierTag.h"
#include "GeoModelKernel/GeoPhysVol.h"
#include "GeoModelKernel/GeoMaterial.h"
#include "GeoModelKernel/GeoTransform.h"
#include "GeoModelKernel/GeoAlignableTransform.h"
#include "InDetReadoutGeometry/PixelDetectorManager.h"
#include "Identifier/Identifier.h"

// For the Msg Svc
#include "AthenaKernel/getMessageSvc.h"
#include "GaudiKernel/MsgStream.h"

GeoVPhysVol* GeoPixelEndCap::Build( ) {

  MsgStream log(Athena::getMessageSvc(), "PixelDetectorFactory");

  //
  // create the Barrel Mother volume
  // 
  double rmin = gmt_mgr->PixelEndcapRMin();
  double rmax = gmt_mgr->PixelEndcapRMax();
  double halflength = (gmt_mgr->PixelEndcapZMax()-gmt_mgr->PixelEndcapZMin())/2.;
  GeoMaterial* air = mat_mgr->getMaterial("std::Air");
  const GeoTube* ecTube = new GeoTube(rmin,rmax,halflength);
  const GeoLogVol* ecLog = new GeoLogVol("EndCapLog",ecTube,air);
  GeoPhysVol* ecPhys = new GeoPhysVol(ecLog);
  //
  // Place the disks and cables on both sides
  //
  GeoPixelDisk pd;
  GeoPixelECCable pecc;
  for(int ii = 0; ii < gmt_mgr->PixelEndcapNDisk(); ii++) {
    gmt_mgr->SetCurrentLD(ii);
    if(gmt_mgr->isLDPresent()) {
      //
      // The position is given w.r.t to the center of the detector!
      //
      // position of the endcap
      double ecz = (gmt_mgr->PixelEndcapZMax()+gmt_mgr->PixelEndcapZMin())/2.;
      //position of the disk
      double zdisk = gmt_mgr->PixelDiskPosition()-ecz;
      // place the disk
      Hep3Vector pos(0.,0.,zdisk);
      GeoNameTag* tag = new GeoNameTag("Disk");
      GeoAlignableTransform* xform = new GeoAlignableTransform(HepTransform3D(HepRotation(), pos));
      GeoVPhysVol * diskPhys = pd.Build();
      ecPhys->add(tag);
      ecPhys->add(new GeoIdentifierTag(ii));
      ecPhys->add(xform);
      ecPhys->add(diskPhys);

      // Store the alignable transform
      int brl_ec = 2*gmt_mgr->GetSide();
      Identifier id = DDmgr->getIdHelper()->wafer_id(brl_ec,ii,0,0);
      DDmgr->addAlignableTransform(1, id, xform, diskPhys);

      //
      // place the cables twice for the two active parts
      //
      double dz = pd.Thickness()/2. + gmt_mgr->PixelECCablesDistance() ;
      pos = Hep3Vector(0.,0.,zdisk+dz);
      tag = new GeoNameTag("ECCables");
      GeoTransform * xformCablesPlus = new GeoTransform(HepTransform3D(HepRotation(), pos));
      ecPhys->add(tag);
      ecPhys->add(xformCablesPlus);
      ecPhys->add(pecc.Build() );
      pos = Hep3Vector(0.,0.,zdisk-dz);
      tag = new GeoNameTag("ECCables");
      GeoTransform * xformCablesMinus = new GeoTransform(HepTransform3D(HepRotation(), pos));
      ecPhys->add(tag);
      ecPhys->add(xformCablesMinus);
      ecPhys->add(pecc.Build() );
    } else {

      log << MSG::DEBUG << "Disk " << ii << " not built" << endreq;

    }
  }
  if(gmt_mgr->DoServices() ) {
    //
    // Add the Endcap services inide the EC volume
    //
    GeoPixelServices ecsvc = GeoPixelServices("/Pixel/PixelEndcap");
    for(int ii =0; ii< ecsvc.NCylinders(); ii++) {
      ecsvc.SetCylinder(ii);
      GeoNameTag* tag = new GeoNameTag("Outside Endcap Service");
      Hep3Vector pos(0.,0.,ecsvc.ZPos() );
      GeoTransform* xform = new GeoTransform(HepTransform3D(HepRotation(),pos));
      ecPhys->add(tag);
      ecPhys->add(xform);
      ecPhys->add(ecsvc.Build() );
    } 
  }
 return ecPhys;
}
