#include "PixelGeoModel/GeoPixelEnvelope.h"
#include "GeoModelKernel/GeoTube.h"
#include "GeoModelKernel/GeoLogVol.h"
#include "GeoModelKernel/GeoNameTag.h"
#include "GeoModelKernel/GeoIdentifierTag.h"
#include "GeoModelKernel/GeoFullPhysVol.h"
#include "GeoModelKernel/GeoTransform.h"
#include "GeoModelKernel/GeoMaterial.h"

#include "PixelGeoModel/GeoPixelBarrel.h"
#include "PixelGeoModel/GeoPixelBcm.h"
#include "PixelGeoModel/GeoPixelEndCap.h"
#include "PixelGeoModel/GeoPixelServices.h"
#include "PixelGeoModel/GeoPixelLadder.h"
#include "PixelGeoModel/GeoPixelModule.h"
#include "PixelGeoModel/GeoPixelTubeCables.h"

GeoVPhysVol* GeoPixelEnvelope::Build( ) {
  //
  // create the External volume where I'll put all the pixel detector
  // 
  double rmin = gmt_mgr->PixelRMin();
  double rmax = gmt_mgr->PixelRMax();
  double halflength = gmt_mgr->PixelHalfLength();
  //std::cout << "Material Manager " << mat_mgr << std::endl;
  GeoMaterial* air = mat_mgr->getMaterial("std::Air");
  const GeoTube* envelopeTube = new GeoTube(rmin,rmax,halflength);
  const GeoLogVol* envelopeLog = new GeoLogVol("Pixel",envelopeTube,air);
  GeoFullPhysVol* envelopePhys = new GeoFullPhysVol(envelopeLog);
  //
  // Add the Barrel:
  //
 gmt_mgr->SetBarrel();
  GeoPixelBarrel brl;
  GeoNameTag* tag = new GeoNameTag("Barrel");
  GeoVPhysVol* barrelPhys =  brl.Build() ;
  envelopePhys->add(tag);
  envelopePhys->add(new GeoIdentifierTag(0));
  envelopePhys->add(barrelPhys );
  if(gmt_mgr->DoServices() ) {
    //
    // Add the Barrel services outside the barrel volume
    //
    GeoPixelServices brlsvc = GeoPixelServices("/Pixel");
    for(int ii =0; ii< brlsvc.NCylinders(); ii++) {
      brlsvc.SetCylinder(ii);
      GeoNameTag* tag = new GeoNameTag("Outside Barrel Service");
      Hep3Vector pos(0.,0.,brlsvc.ZPos() );
      GeoTransform* xform = new GeoTransform(HepTransform3D(HepRotation(),pos));
      envelopePhys->add(tag);
      envelopePhys->add(new GeoIdentifierTag(ii) );
      envelopePhys->add(xform);
      envelopePhys->add(brlsvc.Build() );
    }
  }
 


 //Add BCM Detectors
 
GeoPixelBcm bcm;
//double bcmRadius=56.39;
//double bcmZ=1838+1.85;
double cruciformZ=1838;
double bcmTilt=45.*deg;

//double bcmModRadius=56.39+59.7845*std::cos(bcmTilt) - 4.1265*std::cos(bcmTilt);
//double bcmModZ=cruciformZ+1.85+59.7845*std::sin(bcmTilt) + 4.1265*std::sin(bcmTilt);

double bcmModRadius=56.39+59.7845*std::cos(bcmTilt) - 5.6265*std::cos(bcmTilt);
double bcmModZ=cruciformZ+1.85+59.7845*std::sin(bcmTilt) + 5.6265*std::sin(bcmTilt);


  for(int j=-1;j<=1;j=j+2) {
	for (int i=0;i<4;i++) {
		double phi, x, y, z;
    		phi = 360.*deg/(4)*i;
   		 x = bcmModRadius*std::cos(phi);
   		 y = bcmModRadius*std::sin(phi);
   		 z = j*bcmModZ;
		Hep3Vector pos(x,y,z);
		HepRotation rm;
		if (j==-1)
			rm.rotateX(180.*deg);
 		rm.rotateY(j*bcmTilt);
    		rm.rotateZ(phi);
    		GeoTransform* xform = new GeoTransform(HepTransform3D(rm,pos));

		int k=0;
		if (j==-1)
			k=i+4;
		else if (j==1)
			k=i;

		
  		GeoNameTag* tag = new GeoNameTag("BCM Module");
		GeoVPhysVol* bcmModPhys =  bcm.Build() ;
		envelopePhys->add(tag);
		envelopePhys->add(new GeoIdentifierTag(k));
  		envelopePhys->add(xform);
		envelopePhys->add(bcmModPhys );		
	}
   }




 //
 // Add the EndCaps
  //
  gmt_mgr->SetEndcap();
  gmt_mgr->SetPos();
  GeoPixelEndCap pec;
  double zpos = (gmt_mgr->PixelEndcapZMax()+gmt_mgr->PixelEndcapZMin())/2.;
  Hep3Vector pos(0.,0.,zpos);
  GeoTransform* xform = new GeoTransform(HepTransform3D(HepRotation(),pos));
  tag  = new GeoNameTag("EndCap 1");
//GeoNameTag* tag = new GeoNameTag("EndCap 1");
  envelopePhys->add(tag);
  envelopePhys->add(new GeoIdentifierTag(2));
  envelopePhys->add(xform);
  envelopePhys->add(pec.Build() );
  gmt_mgr->SetNeg();
  pos = Hep3Vector(0.,0.,-zpos);
  HepRotation rm;
  rm.rotateX(180.*deg);
  xform = new GeoTransform(HepTransform3D(rm,pos));
  tag  = new GeoNameTag("EndCap 2");
  envelopePhys->add(tag);
  envelopePhys->add(new GeoIdentifierTag(-2));
  envelopePhys->add(xform);
  envelopePhys->add(pec.Build() );
  if(gmt_mgr->DoServices() ) {
    //
    // Add the Endcap services outside the EC volume
    //
    GeoPixelServices ecsvc = GeoPixelServices("/Pixel");
    for(int ii =0; ii< ecsvc.NCylinders(); ii++) {
      ecsvc.SetCylinder(ii);
      GeoNameTag* tag = new GeoNameTag("Outside Endcap Service");
      Hep3Vector pos(0.,0.,ecsvc.ZPos() );
      GeoTransform* xform = new GeoTransform(HepTransform3D(HepRotation(),pos));
      envelopePhys->add(tag);
      envelopePhys->add(xform);
      envelopePhys->add(new GeoIdentifierTag(ii) );
      envelopePhys->add(ecsvc.Build() );
    }
  } 

 
  



  int nechelles = gmt_mgr->NStave();
  int nmodules  = nechelles*gmt_mgr->PixelNModule();
  std::cout << "-----oooOOOO00000000OOOOOOooooo------------------------"<<std::endl;
  std::cout << "Number of Staves "<<nechelles<<", of Modules "<< nmodules<<std::endl;
  //if(gmt_mgr->DoServices() ) {
    std::cout << "A conn/pig/t0        weighs "<<gmt_mgr->WeiCable()/nechelles<<" grams"<<std::endl;
    std::cout << "An Omega+Tube        weighs "<<gmt_mgr->WeiOmega()/nechelles<<" grams"<<std::endl;
    //}
  std::cout << "A Ladder             weighs "<<gmt_mgr->WeiTrapTmt()/nechelles<<" grams"<<std::endl;
  std::cout << "An average Fluid/lad weighs "<<gmt_mgr->WeiFluid()/nechelles<<" grams"<<std::endl;
  std::cout << "A Module             weighs "<<gmt_mgr->WeiModule()/nmodules<<" grams"<<std::endl;
  std::cout <<"-----oooOOOO00000000OOOOOOooooo------------------------"<<std::endl;
  std::cout << "The Pixel Detector Weight is : "<<gmt_mgr->LePoids()<<" grams"<<std::endl;
  std::cout <<"-----oooOOOO00000000OOOOOOooooo------------------------"<<std::endl;

  return envelopePhys;

  //
  // Want other stuff?
  //
  //  GeoPixelLadder brl;
  //  GeoPixelTubeCables brl;
  // GeoPixelModule brl;


}


