//
// This class builds one Hybrid to be placed in one module
//
// The information on which layer to build is retrieved by the Geometry Manager
//
// Contained by: PixelModule_log
//
// Contains: nothing
//
#include "PixelGeoModel/GeoPixelHybrid.h"
#include "GeoModelKernel/GeoBox.h"
#include "GeoModelKernel/GeoLogVol.h"
#include "GeoModelKernel/GeoNameTag.h"
#include "GeoModelKernel/GeoPhysVol.h"
#include "GeoModelKernel/GeoMaterial.h"

GeoVPhysVol* GeoPixelHybrid::Build() {
  //
  // Dimensions
  //
  double length = gmt_mgr->PixelHybridLength();
  double thickness = gmt_mgr->PixelHybridThickness();
  double width = gmt_mgr->PixelHybridWidth();
  GeoMaterial* hybridMat = mat_mgr->getMaterial("pix::Hybrid");
  if(gmt_mgr->isEndcap() )hybridMat =  mat_mgr->getMaterial("std::Silicon");
  const GeoBox* hybridBox = new GeoBox(thickness/2.,width/2.,length/2.);
  std::string logName = "hybridECLog";
  if(gmt_mgr->isBarrel() ) logName = "hybridBrlLog";
  GeoLogVol* theHybrid = new GeoLogVol(logName,hybridBox,hybridMat);
  GeoPhysVol* hybridPhys = new GeoPhysVol(theHybrid);
  //
  // Add its weight to the full detector
  //
  double weihybrid = hybridBox->volume()*hybridMat->getDensity()/gram;
  //  std::cout<<"And the Hybrid Weight is "<<weihybrid<<" grams"<<std::endl;
  gmt_mgr->AddWeight(weihybrid);
  gmt_mgr->AddWeightModule(weihybrid);
  //
  return hybridPhys;
}
