#include "PixelGeoModel/GeoPixelLayer.h"
#include "PixelGeoModel/GeoPixelLadder.h"
//#include "PixelGeoModel/GeoPixelTubeCables.h"
#include "PixelGeoModel/GeoPixelTubeOmega.h"
#include "PixelGeoModel/GeoPixelSiCrystal.h"

#include "GeoModelKernel/GeoTube.h"
#include "GeoModelKernel/GeoLogVol.h"
#include "GeoModelKernel/GeoNameTag.h"
#include "GeoModelKernel/GeoIdentifierTag.h"
#include "GeoModelKernel/GeoFullPhysVol.h"
#include "GeoModelKernel/GeoMaterial.h"
#include "GeoModelKernel/GeoTransform.h"

#include "GeoModelKernel/GeoTubs.h"
#include "GeoModelKernel/GeoPhysVol.h"


GeoVPhysVol* GeoPixelLayer::Build() {
  //
  // create the three barrel layers
  // Build the sensor first to use the same for all the module in the layer
  bool isBLayer = false;
  if(gmt_mgr->GetLD() == 0) isBLayer = true;
  GeoPixelSiCrystal theSensor(isBLayer);
  //
  // This is the maximum possible w/o going out of the mother volume!
  //
  //double LayerThickness = 8.499*mm;
  //JBdV Try another way
  GeoPixelLadder pl(theSensor);
  double layerradiusMarge = gmt_mgr->PixelLayerRadius();
  //double thicknessMarge   = gmt_mgr->PixelLadderThickness()+gmt_mgr->PixelCableThickness();
  double thicknessMarge   = gmt_mgr->PixelLadderThickness()+gmt_mgr->PixelCableThickness() + 2.5; // plus 0.25 cm New DC3 ???
  double largep           = pl.Thickness()/2.+thicknessMarge;
  double largen           = pl.Thickness()/2.;
  double width            = std::max(std::max(gmt_mgr->PixelBoardWidth(),gmt_mgr->PixelHybridWidth()),
				     gmt_mgr->PixelChipWidth())/2.;
  double ct = cos(gmt_mgr->PixelLadderTilt());
  double st = sin(gmt_mgr->PixelLadderTilt());
  double xb = ct*largep - st*width + layerradiusMarge;
  double yb = st*largep + ct*width;
  double r2 = sqrt(xb*xb + yb*yb);
  double LayerThicknessp = r2 - layerradiusMarge;
  xb = -ct*largen + st*width + layerradiusMarge;
  yb = -st*largen - ct*width;
  r2 = sqrt(xb*xb + yb*yb);
  double LayerThicknessn = layerradiusMarge - r2;
  std::cout<<"Safety margins : "<<LayerThicknessp<<" and "<<LayerThicknessn
	   <<" instead of 8.499 mm "<<std::endl;
  //
  //
  GeoMaterial* air = mat_mgr->getMaterial("std::Air");
  //
  // Layer dimensions from the geometry manager
  //
  double rmin =  gmt_mgr->PixelLayerRadius()-LayerThicknessn;
  double rmax =  gmt_mgr->PixelLayerRadius()+LayerThicknessp;
  double halflength = gmt_mgr->PixelLadderHalfLength() + m_epsilon;
  std::string lname[3];
  lname[0] = "InnerLayerLog";
  lname[1] = "CenterLayerLog";
  lname[2] = "OuterLayerLog";
  const GeoTube* layerTube = new GeoTube(rmin,rmax,halflength); //solid
  const GeoLogVol* layerLog = new GeoLogVol(lname[gmt_mgr->GetLD()],layerTube,air); //log volume
  GeoFullPhysVol* layerPhys = new GeoFullPhysVol(layerLog); // phys vol
  //
  // Place the ladders:
  //
  //std::cout <<"Ladder Thickness " << pl.Thickness() << " InnerLayer halflength "<<halflength<<std::endl;
  //GeoPixelTubeCables ptc;
  GeoPixelTubeOmega ptc;
  int nsectors = gmt_mgr->NPixelSectors();
  gmt_mgr->AddNStaves((unsigned int) nsectors);
  double widthtube   = gmt_mgr->PixelLadderWidth();
  double t1,t2,t12,w12;
  //double thickness = gmt_mgr->PixelLadderThickness()+gmt_mgr->PixelCableThickness();
  double thickness = gmt_mgr->PixelLadderThickness()+gmt_mgr->PixelCableThickness() + 2.5;  // plus 0.25 cm New DC3 ???
  t12 = (pl.Thickness()+thickness)/2.;
  w12 = (widthtube-gmt_mgr->PixelTrapWidth())/2.;
  t1  = t12*cos(gmt_mgr->PixelLadderTilt()) + w12*sin(gmt_mgr->PixelLadderTilt())+m_epsilon;
  t2  = t12*sin(gmt_mgr->PixelLadderTilt()) - w12*cos(gmt_mgr->PixelLadderTilt())+m_epsilon;

  double angle=360./nsectors*deg;
  double layerradius = gmt_mgr->PixelLayerRadius();
  //double xcblpos =  layerradius + (pl.Thickness()/2.+ptc.Thickness()/2)/cos(gmt_mgr->PixelLadderTilt());
  Hep3Vector posladder(layerradius, 0.,0.);
  //Hep3Vector postubecables(xcblpos, 0.,0.);
  Hep3Vector postubecables(layerradius+t1, t2,0.);
  posladder.rotateZ(angle/2.);
  postubecables.rotateZ(angle/2.);
//
// Place the volumes by rotation.
//
  for(int ii = 0; ii < nsectors; ii++) {
    gmt_mgr->SetPhi(ii);
    HepRotation rm;
    //
    // change the sign w.r.t G4 (same sign as G3)
    //
    rm.rotateZ(( +(float) ii+0.5)*angle+gmt_mgr->PixelLadderTilt() ); // pourquoi ce 0.5 ??
    //
    // Place the ladders
    //
    GeoNameTag *tag = new GeoNameTag("Ladder");         
    GeoTransform* xform = new GeoTransform(HepTransform3D(rm,posladder));
    layerPhys->add(tag);
    layerPhys->add(new GeoIdentifierTag(ii) );
    layerPhys->add(xform);
    GeoVPhysVol* ladderphys = pl.Build() ;
    layerPhys->add(ladderphys );
    posladder.rotateZ(angle);     
    if(gmt_mgr->DoServicesOnLadder() ) {
      //
      // Place the box w/ the tubes and cables for the layer
      //
      tag = new GeoNameTag("TubesAndCables");         
      xform = new GeoTransform(HepTransform3D(rm,postubecables));
      layerPhys->add(tag);
      layerPhys->add(new GeoIdentifierTag(ii) );
      layerPhys->add(xform);
      GeoVPhysVol* TCphys = ptc.Build() ;
      layerPhys->add(TCphys );
      for (int kk = 0; kk<gmt_mgr->PixelNModule(); kk++) {
	int indexz    = abs(kk-6);
	double zshift = gmt_mgr->PixelModulePosition(indexz);
	if (kk < 6) zshift = -zshift;
	for (int jj = 0; jj < gmt_mgr->PixelConnectorBNCyliEl(); jj++) {
	  gmt_mgr->SetCurrentCyliEl(jj);
	  int imat      = gmt_mgr->PixelMaterialCyli("Pobi");
	  double rmin   = gmt_mgr->PixelPotiRmin("Pobi");
	  double rmax   = gmt_mgr->PixelPotiRmax("Pobi");
	  double length = gmt_mgr->PixelPotiHalfLength("Pobi");
	  double phi1   = gmt_mgr->PixelPotiPhi1("Pobi");
	  double dphi   = gmt_mgr->PixelPotiPhi2("Pobi")-phi1;
	  std::string logName = "CyliConnecB";
	  std::ostringstream o;
	  o << gmt_mgr->PixelPotiLabel("Pobi");
	  logName = logName+o.str();
	  GeoMaterial* cyliMat = mat_mgr->getMaterial(gmt_mgr->PixelGetMat(imat));
	  const GeoTubs* tubs  = new GeoTubs(rmin,rmax,length,phi1,dphi);
	  GeoLogVol* theTubs   = new GeoLogVol(logName,tubs,cyliMat);
	  GeoPhysVol* pobiphys = new GeoPhysVol(theTubs);
	  GeoNameTag* tag      = new GeoNameTag("ConnecCyli"); 
	  double xpos = gmt_mgr->PixelPotiPosx("Pobi") - thickness - 0.*mm; // Test;
	  double ypos = gmt_mgr->PixelPotiPosy("Pobi") - 2.*mm; // Test
	  double zpos = gmt_mgr->PixelPotiPosz("Pobi") + zshift;
	  Hep3Vector pobipos(xpos,ypos,zpos);
	  GeoTransform* xform = new GeoTransform(HepTransform3D(rm,pobipos+postubecables));
	  //std::cout<<"Adding  the "<<gmt_mgr->PixelPotiLabel("Pobi") <<"th cyli (pobi) "<< ii << " " << kk << " "  << jj <<std::endl;
	  //JBdV je ne les mets pas pour le moment !
// 	  layerPhys->add(tag);
// 	  layerPhys->add(new GeoIdentifierTag(ii));
// 	  layerPhys->add(xform);
// 	  layerPhys->add(pobiphys);
	  // Add the weight (cheks)
	  double weicyli = tubs->volume()*cyliMat->getDensity()/gram;
	  //std::cout<<"And the "<<gmt_mgr->PixelPotiLabel("Pobi") <<"th cyli (pobi) Weight is "<<weicyli<<" grams"<<std::endl;
	  gmt_mgr->AddWeight(weicyli);
	  gmt_mgr->AddWeightCable(weicyli);
	}
      }
      postubecables.rotateZ(angle);
    }
  }
  return layerPhys;
}
