//
// This class builds one parallelepipedic element for the ladder Services
//
// Contains: nothing
//
#include "PixelGeoModel/GeoPixelPosi.h"
#include "GeoModelKernel/GeoPara.h"
#include "GeoModelKernel/GeoLogVol.h"
#include "GeoModelKernel/GeoPhysVol.h"
#include "GeoModelKernel/GeoMaterial.h"

#include<math.h>
#include <sstream>

GeoVPhysVol* GeoPixelPosi::Build() {
  std::string logName, matName;
  //
  int label = gmt_mgr->PixelPosiLabel();
  int imat  = gmt_mgr->PixelMaterialPara();
  //
  // Dimensions
  //
  double theta  = 0.;
  double phi    = 0.;
  double alpha  = gmt_mgr->PixelPosiTilt();
  double length = gmt_mgr->PixelPosiHalfLength();
  double width  = gmt_mgr->PixelPosiWidth();
  double thick  = gmt_mgr->PixelPosiThick();
  if (gmt_mgr->GetParaElType() == "Posi") {
    logName ="ParaTub";
  } else if (gmt_mgr->GetParaElType() == "Poai") {
    logName ="ParaConnecA";
  } else if (gmt_mgr->GetParaElType() == "Poci") {
    logName ="ParaConnecC";
  }
  std::ostringstream o;
  o << label;
  logName = logName+o.str();
  GeoMaterial* paraMat = mat_mgr->getMaterial(gmt_mgr->PixelGetMat(imat));
  const GeoPara* para  = new GeoPara(thick/2.,width/2.,length,alpha,theta,phi);
  GeoLogVol* thePara   = new GeoLogVol(logName,para,paraMat);
  GeoPhysVol* paraPhys = new GeoPhysVol(thePara);
  //
  double weipara = para->volume()*paraMat->getDensity()/gram;
  //std::cout<<"And the "<<label<<"th para of type "<< gmt_mgr->GetParaElType() <<" Weight is "<<weipara<<" grams" <<std::endl;
  gmt_mgr->AddWeight(weipara);
  if (gmt_mgr->GetParaElType() == "Posi") {
    //std::cout<<"And the "<<label<<"th para of type "<< gmt_mgr->GetParaElType() <<" Weight is "<<weipara<<" grams and the volume "<<para->volume() <<std::endl;
    gmt_mgr->AddWeightOmega(weipara);
  } else if (gmt_mgr->GetParaElType() == "Poai" || gmt_mgr->GetParaElType() == "Poci") {
    gmt_mgr->AddWeightCable(weipara);
  }

  //
  return paraPhys;
}
