#include "PixelGeoModel/GeoPixelSubDisk.h"
#include "PixelGeoModel/GeoPixelChip.h"
#include "PixelGeoModel/GeoPixelHybrid.h"
#include "PixelGeoModel/GeoPixelSiCrystal.h"
#include "GeoModelKernel/GeoTubs.h"
#include "GeoModelKernel/GeoLogVol.h"
#include "GeoModelKernel/GeoNameTag.h"
#include "GeoModelKernel/GeoIdentifierTag.h"
#include "GeoModelKernel/GeoFullPhysVol.h"
#include "GeoModelKernel/GeoMaterial.h"
#include "GeoModelKernel/GeoTransform.h"
#include "GeoModelKernel/GeoAlignableTransform.h"

GeoPixelSubDisk::GeoPixelSubDisk(GeoPixelSiCrystal & theSensor) :
m_theSensor(theSensor)
{
  //
  // Define the log volume in the constructor, so I do it only once.
  //

  //   
  // Dimensions from class methods.
  //
  double rmin = RMin(); 
  double rmax = RMax();
  double halflength = Thickness()/2.;
  GeoMaterial* air = mat_mgr->getMaterial("std::Air");
  const GeoTubs* SDTubs = new GeoTubs(rmin,rmax,halflength,-180.*deg/gmt_mgr->PixelECNSectors1()+0.000005,360.*deg/gmt_mgr->PixelECNSectors1()-0.00001);
  theSubDisk = new GeoLogVol("SubDiskLog",SDTubs,air);
}

GeoVPhysVol* GeoPixelSubDisk::Build( ) {
  GeoFullPhysVol* SDPhys = new GeoFullPhysVol(theSubDisk);
  //
  // Place the Si Crystal
  //
  double xpos = RMin()+gmt_mgr->PixelBoardLength()/2.;
  GeoNameTag* tag = new GeoNameTag("SiCrystal");
  HepRotation rm;
  rm.rotateY(90.*deg);
  rm.rotateX(180.*deg);
  Hep3Vector pos(xpos,0.,0.);
  GeoAlignableTransform* xformsi = new GeoAlignableTransform(HepTransform3D(rm,pos) );
  SDPhys->add(tag);
  SDPhys->add(new GeoIdentifierTag(200) );
  SDPhys->add(xformsi);
  SDPhys->add(m_theSensor.Build() );
  //
  // Place the Hybrid
  //
  GeoPixelHybrid ph;
  tag = new GeoNameTag("Hybrid");
  double zpos = 0.5*(gmt_mgr->PixelBoardThickness()+gmt_mgr->PixelHybridThickness())+m_epsilon/2.;
  pos = Hep3Vector(xpos,0.,zpos);
  GeoTransform* xform = new GeoTransform(HepTransform3D(rm,pos) );
  SDPhys->add(tag);
  SDPhys->add(xform);
  SDPhys->add(ph.Build() );
//
// Place the Chip
//
  GeoPixelChip pc;
  tag = new GeoNameTag("Chip");
  zpos = -0.5*(gmt_mgr->PixelBoardThickness()+gmt_mgr->PixelChipThickness())-gmt_mgr->PixelChipGap();
  pos = Hep3Vector(xpos,0.,zpos);
  xform = new GeoTransform(HepTransform3D(rm,pos) );
  SDPhys->add(tag);
  SDPhys->add(xform);
  SDPhys->add(pc.Build() );
  return SDPhys;
}


double GeoPixelSubDisk::RMin() {
  return gmt_mgr->PixelDiskRMin()-0.5*(gmt_mgr->PixelBoardLength()-gmt_mgr->PixelBoardActiveLen());
}

double GeoPixelSubDisk::RMax() {
  return RMin()+sqrt(pow(gmt_mgr->PixelBoardLength(),2)+pow(gmt_mgr->PixelBoardWidth(),2));
}

double GeoPixelSubDisk::Thickness() {
  //
  // This is the sum of the Thickness of the board + twice the maximum
  // b/w the thickness of the hybrid and that of the electronics
  // twice for symmetric placement.
  //

  return gmt_mgr->PixelBoardThickness()+2*std::max(gmt_mgr->PixelHybridThickness(),gmt_mgr->PixelChipThickness())+2*m_epsilon;
}

