//
// This class builds one trapezoid
//
//
#include "PixelGeoModel/GeoPixelTrapTmt.h"
#include "GeoModelKernel/GeoTrap.h"
#include "GeoModelKernel/GeoLogVol.h"
#include "GeoModelKernel/GeoNameTag.h"
#include "GeoModelKernel/GeoPhysVol.h"
#include "GeoModelKernel/GeoMaterial.h"

#include<math.h>
#include <sstream>

GeoVPhysVol* GeoPixelTrapTmt::Build() {
  //
  // Dimensions
  //
  double al1 = 0.;
  double al2 = 0.;
  double phi = 0.;
  double length   = gmt_mgr->PixelTrapHalfLength();
  double widthy   = gmt_mgr->PixelTrapHalfWidthY();
  double widthxzn = gmt_mgr->PixelTrapHalfWidthXzn();
  double widthxzp = gmt_mgr->PixelTrapHalfWidthXzp();
  double tet;
  if (gmt_mgr->PixelTrapDzdr() == 0 && gmt_mgr->GetTrapType() == "TMT") {
    tet = atan2(widthxzn-widthxzp,2.*length)*rad;
  } else {
    tet = gmt_mgr->PixelTrapDzdr() ;
  }
  std::string logName;
  int imat = 8;
  if (gmt_mgr->GetTrapType() == "TMT") {
    if (gmt_mgr->GetTrap() < gmt_mgr->PixelNTrapC()) {
      logName = "trapCLog";
    } else if (gmt_mgr->GetTrap() < gmt_mgr->PixelNTrapC() + gmt_mgr->PixelNTrapE() ) {
      logName = "trapELog";
    } else {
      logName = "trapLog";
    }
  } else {
    logName = "trapFLog";
    imat    = gmt_mgr->PixelTrapMat()+imat;
  } 
  std::ostringstream o;
  o << gmt_mgr->PixelTrapLabel();
  logName = logName+o.str();
  const GeoMaterial* trapMat = mat_mgr->getMaterial(gmt_mgr->PixelGetMat(imat));
  const GeoTrap* trap  = new GeoTrap(length,tet,phi,widthy,widthxzn,widthxzn,al1,widthy,widthxzp,widthxzp,al2);
  GeoLogVol* theTrap   = new GeoLogVol(logName,trap,trapMat);
  GeoPhysVol* trapPhys = new GeoPhysVol(theTrap);
  //
  // Add its weight to the full detector
  //
  double weitmt = trap->volume()*trapMat->getDensity()/gram;
  //std::cout<<"And the "<<gmt_mgr->PixelTrapLabel()<<"th trap Weight is "<<weitmt<<" grams"<<std::endl;
  gmt_mgr->AddWeight(weitmt);
  if (gmt_mgr->GetTrapType() == "TMT") gmt_mgr->AddWeightTrapTmt(weitmt);
  if (gmt_mgr->GetTrapType() == "Fluid") gmt_mgr->AddWeightFluid(weitmt);
  //
  return trapPhys;
}
